#!/usr/bin/env python2.5
# coding= utf-8
# 
# Copyright (c) 2009 Daniel Would
# Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#

# ============================================================================
# Name        : gtkWitter.py
# Author      : Daniel Would
# Version     : 0.1
# Description : Witter ui code, gtk
# ============================================================================

#This is the bunch of things I wound up importing
#I think I need them all.. 
import gtk
import pygtk
import hildon

import re
import string
import osso
import os
import witter
import account

#The Witter UI class
class WitterUI():
    #first an init method to set everything up    
    TIMELINE_VIEW = 1
    MENTIONS_VIEW = 2
    DM_VIEW = 3
    SEARCH_VIEW = 4
    FRIENDS_VIEW = 5
    TRENDS_VIEW = 6
    PUBLIC_VIEW = 7
    USERHIST_VIEW = 8
    def __init__(self, controller):
        self.version = controller.getVersion()
        self.controller = controller
        #controller is the master program which handles all the events
        # set name of application: this shows in titlebar
        gtk.set_application_name("Witter")
        self.serviceName = "Witter"

        # 
        #we want to track our position in each tweet stream so we can 
        #return to it when we switch between them
        self.timelinePos = None
        self.mentionPos = None
        self.dmPos = None
        self.searchPos = None
        self.friendsPos = None
        self.trendsPos = None
        self.publicPos = None
        self.showingMore = False
        self.busyCounter = 0

        #Set the Glade file
        #self.gladefile = "/usr/share/witter/witter.glade"  
        #self.wTree = gtk.glade.XML(self.gladefile) 
        self.builder = gtk.Builder()
        self.builder.add_from_file("/usr/share/witter/witter.ui")
        #map all the signals
        dic = {
            "newTweet" : controller.enterPressed,
            "getTweets" : controller.updateSelectedView,
            "storecreds" : controller.store_creds,
            "getacesstoken" : controller.getAccessToken,
            "on_timeline_clicked" : self.switchView,
            "on_mentions_clicked" : self.switchView,
            "on_direct_messages_clicked" : self.switchView,
            "on_search_clicked" : self.switchView,
            "on_trend_clicked" : self.switchView,
            "on_user_history_clicked" : self.switchView,
            "on_insert_clicked" : controller.twitPic,
            "on_friends_clicked" : self.switchView,
            "on_trends_clicked" : self.switchView,
            "on_public_clicked" : self.switchView,
            "on_Reply_clicked" : controller.replyTo,
            "on_ReplyAll_clicked" : controller.replyAll,
            "on_AtUser_clicked" : controller.tweetAt,
            "on_Retweet_clicked" : controller.reTweet,
            "on_DirectMessage_clicked" : controller.directMessage,
            "on_Follow_clicked" : controller.FollowTweetAuthor,
            "on_UnFollow_clicked" : controller.UnFollowTweetAuthor,
            "on_cancel_clicked" : self.gtk_widget_hide,
            "on_fullscr_clicked" : self.fullscr,
            "on_extra_clicked" : self.showMoreButtons,
            "on_plus20_clicked" : controller.get20More,
            "on_plus50_clicked" : controller.get50More,
            "on_plus100_clicked" : controller.get100More,
            "on_plus200_clicked" : controller.get200More,

        }
        self.builder.connect_signals(dic)
        #self.wTree.signal_autoconnect( dic )

        #fix the buttons to get the style right 
        refreshButton = self.builder.get_object("Refresh")
        refImage = gtk.Image()
        refImage.set_from_file("/opt/witter/icons/refresh.png")
        refImage.show()
        refreshButton.set_image(refImage)
        tweetButton = self.builder.get_object("Tweet")
        tweetImage = gtk.Image()
        tweetImage.set_from_file("/opt/witter/icons/tweet.png")
        tweetImage.show()
        tweetButton.set_image(tweetImage)
        timelineButton = self.builder.get_object("timeline")
        tlImage = gtk.Image()
        tlImage.set_from_file("/opt/witter/icons/timeline.png")
        tlImage.show()
        timelineButton.set_image(tlImage)
        mentionsButton = self.builder.get_object("mentions")
        menImage = gtk.Image()
        menImage.set_from_file("/opt/witter/icons/mention_off.png")
        menImage.show()
        mentionsButton.set_image(menImage)
        dmsButton = self.builder.get_object("direct messages")
        dmImage = gtk.Image()
        dmImage.set_from_file("/opt/witter/icons/dm_off.png")
        dmImage.show()
        dmsButton.set_image(dmImage)
        searchButton = self.builder.get_object("search")
        serImage = gtk.Image()
        serImage.set_from_file("/opt/witter/icons/search_off.png")
        serImage.show()
        searchButton.set_image(serImage)
        friendsButton = self.builder.get_object("friends")
        friendImage = gtk.Image()
        friendImage.set_from_file("/opt/witter/icons/friends_off.png")
        friendImage.show()
        friendsButton.set_image(friendImage)
        publicButton = self.builder.get_object("public")
        pubImage = gtk.Image()
        pubImage.set_from_file("/opt/witter/icons/public_off.png")
        pubImage.show()

        publicButton.set_image(pubImage)
        trendsButton = self.builder.get_object("trends")
        trendImage = gtk.Image()
        trendImage.set_from_file("/opt/witter/icons/trends_off.png")
        trendImage.show()
        trendsButton.set_image(trendImage)
        userButton = self.builder.get_object("userhistory")
        userImage = gtk.Image()
        userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
        userImage.show()
        userButton.set_image(userImage)

        fullscrButton = self.builder.get_object("fullscr")
        image = gtk.Image()
        image.set_from_file("/opt/witter/icons/fullscr.png")
        image.show()
        fullscrButton.set_image(image)
        extraButton = self.builder.get_object("extra-controls")
        image2 = gtk.Image()
        image2.set_from_file("/opt/witter/icons/plus.png")
        image2.show()
        extraButton.set_image(image2)


        plus20Button = self.builder.get_object("plus20")
        plus50Button = self.builder.get_object("plus50")
        plus100Button = self.builder.get_object("plus100")
        plus200Button = self.builder.get_object("plus200")


        imageplus20 = gtk.Image()
        imageplus20.set_from_file("/opt/witter/icons/plus20.png")
        imageplus20.show()
        plus20Button.set_image(imageplus20)

        imageplus50 = gtk.Image()
        imageplus50.set_from_file("/opt/witter/icons/plus50.png")
        imageplus50.show()
        plus50Button.set_image(imageplus50)

        imageplus100 = gtk.Image()
        imageplus100.set_from_file("/opt/witter/icons/plus100.png")
        imageplus100.show()
        plus100Button.set_image(imageplus100)

        imageplus200 = gtk.Image()
        imageplus200.set_from_file("/opt/witter/icons/plus200.png")
        imageplus200.show()
        plus200Button.set_image(imageplus200)

        okButton = self.builder.get_object("Ok")
        cancelButton = self.builder.get_object("Cancel")
        okoauthButton = self.builder.get_object("Ok-oauth")



        #action buttons for a tweet
        replybutton = self.builder.get_object("Reply")
        replyAllbutton = self.builder.get_object("ReplyAll")
        retweetbutton = self.builder.get_object("Retweet")
        dmbutton = self.builder.get_object("DirectMessage")
        atUserbutton = self.builder.get_object("AtUser")


        #followbutton = self.builder.get_object("Follow")
        #unfollowbutton = self.builder.get_object("UnFollow")
        #usersTimelinebutton = self.builder.get_object("UsersTimeline")

        refreshButton.set_name("HildonButton-finger")
        tweetButton.set_name("HildonButton-finger")
        timelineButton.set_name("HildonButton-finger")
        mentionsButton.set_name("HildonButton-finger")
        dmsButton.set_name("HildonButton-finger")
        searchButton.set_name("HildonButton-finger")
        friendsButton.set_name("HildonButton-finger")
        trendsButton.set_name("HildonButton-finger")
        publicButton.set_name("HildonButton-finger")
        userButton.set_name("HildonButton-finger")
        fullscrButton.set_name("HildonButton-finger")
        extraButton.set_name("HildonButton-finger")
        plus20Button.set_name("HildonButton-finger")
        plus50Button.set_name("HildonButton-finger")
        plus100Button.set_name("HildonButton-finger")
        plus200Button.set_name("HildonButton-finger")
        okButton.set_name("HildonButton-finger")
        cancelButton.set_name("HildonButton-finger")
        okoauthButton.set_name("HildonButton-thumb")


        #action buttons
        replybutton.set_name("HildonButton-finger")
        replyAllbutton.set_name("HildonButton-finger")
        retweetbutton.set_name("HildonButton-finger")
        dmbutton.set_name("HildonButton-finger")
        atUserbutton.set_name("HildonButton-finger")
        #followbutton.set_name("HildonButton-thumb")
        #unfollowbutton.set_name("HildonButton-thumb")
        #usersTimelinebutton.set_name("HildonButton-thumb")

        self.textcolour = "#FFFFFF"
        #default colours are the ones used in the hildon window buttons
        self.bg_top_color = gtk.gdk.color_parse("#6bd3ff")
        self.bg_bottom_color = gtk.gdk.color_parse("#0075b5")
        self.font_size = 18

        #at one point I had the text different colours
        #I may do again
        self.namecolour = self.textcolour
        self.tweetcolour = self.textcolour

        self.defaultwidth = 790
        #default to colours above, but check if we're on fremantle and change
        #to appropriate colours if we are

        #This being a hildon app we start with a hildon.Window
        self.window = hildon.StackableWindow()
        self.window.set_app_paintable(True)
        self.window.realize()
        #connect the delete event for closing the window
        self.window.connect("delete_event", self.controller.quit)
        #we default to the timeline view
        self.window.set_title("Witter" + " - timeline")
        #add window to self  
        self.controller.program.add_window(self.window)
        #reparent the vbox1 from glade to self.window
        # self.vbox = self.wTree.get_widget("vbox1")
        self.vbox = self.builder.get_object("vbox1")
        #pannedWindow = hildon.PannableArea()
        pannedWindow = self.builder.get_object("pannableArea")
        # hildon.hildon_pannable_area_new_full(mode, enabled, vel_min, vel_max, decel, sps)
        self.window.window.property_change("_HILDON_ZOOM_KEY_ATOM", "XA_INTEGER", 32, gtk.gdk.PROP_MODE_REPLACE, [1])
        #self.scrolled_window = self.wTree.get_widget("scrolled_window")
        self.vbox.reparent(self.window)
        #self.vbox.pack_end(pannedWindow)

        #self.urlmenu = self.build_right_click_menu()
        # create a menu object by calling a method to deine it
        self.menu = self.create_m5_menu(self)
        # add the menu to the window
        self.window.set_app_menu(self.menu)
        #

        # define a liststore we use this to store our tweets and some associated data
        #all the views store basically 'tweet' data in this form
        # the fields are : Name,nameColour,Tweet,TweetColour,id, type, timestamp, replyTo, source
        self.liststore = gtk.ListStore(str, str, str, str, str, str, str, str, str)

        #we want auto-complete of @references 
        #self.tweetText = self.wTree.get_widget("TweetText")
        self.tweetText = self.builder.get_object("TweetText")
        self.tweetText.connect("changed", self.CharsRemaining)
        tweetComplete = gtk.EntryCompletion()
        tweetComplete.set_model(self.controller.activeAccount.getFriendsList())
        tweetComplete.set_text_column(0)
        tweetComplete.set_inline_completion(True)
        tweetComplete.set_minimum_key_length(2)
        self.tweetText.set_completion(tweetComplete)

        # create the TreeView using treestore this is the object which displays the
        # info stored in the liststore
        #self.treeview = gtk.TreeView(self.liststore)
        self.treeview = hildon.GtkTreeView(gtk.HILDON_UI_MODE_NORMAL)

        #self.treeview = hildon.hildon_gtk_tree_view_new(self.liststore)
        self.treeview.set_model(self.liststore)
        # create the TreeViewColumn to display the data, I decided on two colums
        # one for name and the other for the tweet
        #self.tvcname = gtk.TreeViewColumn('Name')
        #self.cell = witter.witter_cell_renderer.witterCellRender()
        #cell_text = gtk.CellRendererText()
        self.cell = gtk.CellRendererText()

        color = self.bg_top_color

        color2 = self.bg_bottom_color


        #self.cell.set_property('backgroundt_r', color.red)
        #self.cell.set_property('backgroundt_g', color.green)
        #self.cell.set_property('backgroundt_b', color.blue)
        #self.cell.set_property('backgroundt_p', color.pixel)
        #self.cell.set_property('backgroundb_r', color2.red)
        #self.cell.set_property('backgroundb_g', color2.green)
        #self.cell.set_property('backgroundb_b', color2.blue)
        #self.cell.set_property('backgroundb_p', color2.pixel)
        #self.cell.set_property('font_size', self.font_size)

        self.tvctweet = gtk.TreeViewColumn('Pango Markup', self.cell, markup=10)

        #Column 0 for the treeview
        renderer = gtk.CellRendererPixbuf()
        renderer.set_property('yalign', 0)
        #renderer.set_property('ypad', 10)
        #renderer.set_property('cell-background-gdk', gtk.gdk.color_parse("#6bd3ff"))

        column = gtk.TreeViewColumn()
        column.pack_start(renderer, True)
        column.add_attribute(renderer, "pixbuf", 9)
        self.treeview.append_column(column)

        #Column 1 for the treeview
        renderer = gtk.CellRendererText()

        self.tvctweet.set_property("expand", True)
        self.cell.set_property('wrap-mode', "word")
        self.cell.set_property('wrap-width', 730)
        self.cell.set_property('size-points', self.controller.font_size)


        self.treeview.append_column(self.tvctweet)
        self.treeview.set_rules_hint(True)
        self.treeview.set_property('enable-grid-lines', True)
        #self.tvctweet = gtk.TreeViewColumn('Tweet')
        # add the two tree view columns to the treeview
        #self.treeview.append_column(self.tvcname)



        # next we add a mapping to the tweet column, again the third field
        # in our list store is the tweet text
        self.tvctweet.add_attribute(self.cell, 'text', 10)
        #self.tvctweet.add_attribute(self.cell, 'timestamp', 6)
        #self.tvctweet.add_attribute(self.cell, 'replyto', 7)
        #self.tvctweet.add_attribute(self.cell, 'source', 8)

        # make it searchable (I found this in an example and thought I might use it
        # but currently I make no use of this setting
        self.treeview.set_search_column(2)
        self.treeview.set_rules_hint(True)

        #self.treeview.set_property('enable-grid-lines', True)

        # Allow sorting on the column. This is cool because no matter what order
        # we load tweets in, we always get a view which is sorted by the tweet id which
        # always increments, so we get them in order

        self.liststore.set_sort_column_id(4, gtk.SORT_DESCENDING)
        # I don't want to accidentally be dragging and dropping rows out of order
        self.treeview.set_reorderable(False)
        #with all that done I add the treeview to the scrolled window
        pannedWindow.add_with_viewport(self.treeview)
        pannedWindow.connect('horizontal-movement', self.gesture)
        pannedWindow.connect('vertical-movement', self.scrolling)
        #self.treeview.connect("button-press-event", self.build_menu, None);
        selection = self.treeview.get_selection()
        #selection.connect('changed', self.build_menu)
        #self.treeview.connect('row-activated', self.build_menu)
        self.treeview.connect('row-activated', self.build_stacked_menu)

        self.window.show_all()
        self.window.connect("key-press-event", self.on_key_press)
        #self.window.connect("button-press-event", self.on_button_pressed)
        self.window.connect("window-state-event", self.on_window_state_change)
        #hide the action buttons
        self.builder.get_object("hbuttonbox-act1").hide_all()
        self.builder.get_object("hbuttonbox-act2").hide_all()
        self.builder.get_object("hbuttonbox-act3").hide_all()
        self.builder.get_object("hbuttonbox-act4").hide_all()
        self.builder.get_object("hbuttonbox-more").hide_all()
        self.activeView = self.TIMELINE_VIEW

    def setActiveListStore(self, liststore):
        liststore.set_sort_column_id(4, gtk.SORT_DESCENDING)
        self.treeview.set_model(None)
        self.treeview.set_model(liststore)

    def create_m5_menu(self, widget):
        #a fairly standard menu create
        #I put in the same options as I have buttons
        # and linked to the same methods
        menu = hildon.AppMenu()


        ShortenUrl = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        ShortenUrl.set_label("Shorten URL")
        # Attach callback to clicked signal
        ShortenUrl.connect("clicked", self.shortenUrl)
        ShortenUrl.show()
        menu.append(ShortenUrl)

        TwitPic = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        TwitPic.set_label("TwitPic!")
        # Attach callback to clicked signal
        TwitPic.connect("clicked", self.controller.selectImage)
        TwitPic.show()
        menu.append(TwitPic)



        Properties = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        Properties.set_label("Preferences")
        # Attach callback to clicked signal
        Properties.connect("clicked", self.show_props_window)
        Properties.show()
        menu.append(Properties)
        Accounts = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        Accounts.set_label("Account Setup")
        # Attach callback to clicked signal
        Accounts.connect("clicked", self.show_account_window)
        Accounts.show()
        menu.append(Accounts)
        About = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        About.set_label("About")
        # Attach callback to clicked signal
        About.connect("clicked", self.about)
        About.show()
        menu.append(About)

        #invert no longer works
        #Invert = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        #Invert.set_label("Invert")
        # Attach callback to clicked signal
        #Invert.connect("clicked", self.flipTextColour)
        #Invert.show()
        #menu.append(Invert)

        #user can hit the big X for exit, no need for it in menu
        #Exit = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
        #Exit.set_label("Exit")
        ## Attach callback to clicked signal
        #Exit.connect("clicked", self.quit)
        #Exit.show()
        #menu.append(Exit)

        menu.show_all()
        return menu

    def on_key_press(self, widget, event, *args):
        #this picks up the press of the full screen key and toggles
        #from one mode to the other
        curView = self.getCurrentView()
        if event.keyval == gtk.keysyms.F7:
            print "zoom in"
            if (self.controller.font_size < 30):
                self.treeview.hide()

                self.controller.font_size = self.controller.font_size + 1
                self.cell.set_property('size-points', self.controller.font_size)
                if (curView == self.TIMELINE_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getTimeline())
                elif (curView == self.DM_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getDmsList())
                elif (curView == self.MENTIONS_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getMentionsList())
                elif (curView == self.PUBLIC_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getPublicList())
                elif (curView == self.TRENDS_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getTrendsList())
                elif (curView == self.FRIENDS_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getFriendsList())
                elif (curView == self.SEARCH_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getSearchList())
                elif (curView == self.USERHIST_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getUserHistoryList())
                self.treeview.show()

        elif event.keyval == gtk.keysyms.F8:
            print "zoom out"
            if (self.controller.font_size > 10):
                self.treeview.hide()

                self.controller.font_size = self.controller.font_size - 1
                self.cell.set_property('size-points', self.controller.font_size)
                if (curView == self.TIMELINE_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getTimeline())
                elif (curView == self.DM_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getDmsList())
                elif (curView == self.MENTIONS_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getMentionsList())
                elif (curView == self.PUBLIC_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getPublicList())
                elif (curView == self.TRENDS_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getTrendsList())
                elif (curView == self.FRIENDS_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getFriendsList())
                elif (curView == self.SEARCH_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getSearchList())
                elif (curView == self.USERHIST_VIEW):
                    self.treeview.set_model(None)
                    self.treeview.set_model(self.controller.activeAccount.getUserHistoryList())
                self.treeview.show()

        elif event.keyval == gtk.keysyms.F6:
             # The "Full screen" hardware key has been pressed 
             if self.window_in_fullscreen:
                 self.window.unfullscreen ()
                 #when we toggle off fullscreen set the cell render wrap
                 #to 500
                 self.cell.set_property('wrap-width', self.defaultwidth)
             else:
                self.window.fullscreen ()
                #when we toggle into fullscreen set the cell render wrap
                #wider
                self.cell.set_property('wrap-width', 630)
        else:
            self.builder.get_object("hbox1").show_all()
            self.builder.get_object("hbox2").show_all()
            self.builder.get_object("hbuttonbox-act1").hide_all()
            self.builder.get_object("hbuttonbox-act2").hide_all()
            self.builder.get_object("hbuttonbox-act3").hide_all()
            self.builder.get_object("hbuttonbox-act4").hide_all()
            self.tweetText.grab_focus()
            self.window.unfullscreen ()



    def enterPressed(self, widget, *args):
        self.newTweet(self, widget, *args)
        #if we were in the search view, we want to restore the search terms after a tweet
        if (self.treeview.get_model() == self.searchliststore):
            self.tweetText.set_text(self.search_terms)


    def on_window_state_change(self, widget, event, *args):
        #this just sets a flag to keep track of what state we're in
       if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
            self.window_in_fullscreen = True
       else:
            self.window_in_fullscreen = False

    def fullscr(self, *args):
        if (self.window_in_fullscreen == True):
            self.window.unfullscreen ()
            fullscrButton = self.builder.get_object("fullscr")
            image = gtk.Image()
            image.set_from_file("/opt/witter/icons/fullscr.png")
            image.show()
            fullscrButton.set_image(image)
        else:
            self.window.fullscreen ()
            fullscrButton = self.builder.get_object("fullscr")
            image = gtk.Image()
            image.set_from_file("/opt/witter/icons/unfullscr.png")
            image.show()
            fullscrButton.set_image(image)



    def build_menu(self, widget, *args):
        #first hide the other controls
        print "building menu"
        self.builder.get_object("hbox1").hide_all()
        self.builder.get_object("hbox2").hide_all()
        #
        #empty the dynamic buttons
        buttons = self.builder.get_object("hbuttonbox-act3").get_children()
        for button in buttons:
            self.builder.get_object("hbuttonbox-act3").remove(button)
            button.destroy()
        buttons2 = self.builder.get_object("hbuttonbox-act4").get_children()
        for button in buttons2:
            self.builder.get_object("hbuttonbox-act4").remove(button)
            button.destroy()
        #we need to track how many buttons we create
        #so we know when to create new horizontal button boxes
        b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
        #b.set_title('Dismiss')
        disImage = gtk.Image()
        disImage.set_from_file("/opt/witter/icons/dismiss.png")
        disImage.show()
        b.set_image(disImage)
        b.connect("clicked", self.dismissActionButtons)
        self.builder.get_object("hbuttonbox-act4").pack_end(b, expand=True)


        #get the selection
        treeselection = self.treeview.get_selection()
        select1, select2 = treeselection.get_selected_rows()
        #entry1, entry2 = self.treeview.get_selection().get_selected()
        #we might one day have more than on element selected, for now we get 1 row
        try:
            #we want to scroll to the right place in pannable area, *after* we've shown conrolls
            #so the x/y need to be in the right scope
            selectedItem = None
            if select2 != None:

                for item in select2[0]:
                    #bring the selected item to the top of the view

                    #we want to access field 2 (0,1,2) which has our Tweet in it
                    entry = select1.get_value(select1.get_iter(item), 2)
                    #and we might as well list the person who provided the url
                    name = select1.get_value(select1.get_iter(item), 0)
                    id = select1.get_value(select1.get_iter(item), 4)
                    selectedItem = item
                    #exclude logged in user from the reply_all string
                    if (re.search(self.controller.activeAccount.getUsername().lower(), name.lower())):
                        self.reply_all = ""
                    else:
                        self.reply_all = name
                    if re.search("http", entry):
                        #convert the string to chunks deliniated on space (we assume the url 
                        #has spaces around it 
                        L = string.split(entry)
                        for word in L :
                            #find the 'word' which is our url
                            if re.search("http", word):
                                url = word

                                b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
                                b.set_title('Open Url')
                                b.set_value(url)
                                b.connect("clicked", self.controller.openBrowser, url)
                                self.builder.get_object("hbuttonbox-act3").pack_start(b, expand=True)
                    if re.search("@", entry):
                        L = string.split(entry)
                        for word in L :
                            if re.search("@", word):
                                user = word
                                if (user != name):
                                    if (re.search(self.controller.activeAccount.getUsername().lower(), user.lower())):
                                        print "excluding self from reply all"
                                    else:
                                        self.controller.reply_all = self.controller.reply_all + " " + user
                                    b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                                    b.set_title('Follow')
                                    b.set_value(user)
                                    b.connect("clicked", self.controller.activeAccount.FollowUser, user)
                                    self.builder.get_object("hbuttonbox-act4").pack_start(b, expand=True)
                                    b2 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)

                                    b2.set_value(user)
                                    b2.connect("clicked", self.controller.tweetAtUser, user)
                                    self.builder.get_object("hbuttonbox-act4").pack_start(b2, expand=True)
                                    b3 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)

                                    b3.set_title('History')
                                    b3.set_value(user)
                                    b3.connect("clicked", self.controller.showHist, user)
                                    self.builder.get_object("hbuttonbox-act4").pack_start(b3, expand=True)


                    #rect = self.treeview.get_background_area(item, self.tvctweet)
                    #x, y = self.treeview.convert_bin_window_to_tree_coords(rect.x, rect.y)
                #only provide a follow button if we aren't in our own timeline (in which case we're already following
                curView = self.getCurrentView()
                if (curView != self.TIMELINE_VIEW):
                    followButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                          hildon.BUTTON_ARRANGEMENT_VERTICAL)
                    followButton.set_title('Follow')
                    followButton.set_value(name)
                    followButton.connect("clicked", self.controller.activeAccount.FollowUser, name)
                    self.builder.get_object("hbuttonbox-act3").pack_start(followButton, expand=True)
                unfollowButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                unfollowButton.set_title('unFollow')
                unfollowButton.set_value(name)
                unfollowButton.connect("clicked", self.controller.activeAccount.UnFollowUser, name)
                self.builder.get_object("hbuttonbox-act3").pack_start(unfollowButton, expand=True)
                histButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                histButton.set_title('History')
                histButton.set_value(name)
                histButton.connect("clicked", self.controller.showHist, name)
                self.builder.get_object("hbuttonbox-act3").pack_start(histButton, expand=True)

                self.builder.get_object("AtUser").set_label(name)

                self.controller.reply_to = id
                self.controller.reply_to_name = name

                self.controller.retweettext = entry
                self.controller.retweetname = name
                self.controller.retweetid = id

        except IndexError:
            print "nothing selected"

        self.builder.get_object("hbuttonbox-act1").show_all()
        self.builder.get_object("hbuttonbox-act2").show_all()
        self.builder.get_object("hbuttonbox-act3").show_all()
        self.builder.get_object("hbuttonbox-act4").show_all()
        self.scrollToItem(selectedItem)

    def build_stacked_menu(self, widget, *args):
        #first hide the other controls
        print "building stacked menu"
        win = hildon.StackableWindow()
        win.set_title("tweet")
        pannedArea = hildon.PannableArea()
        vbox1 = gtk.VBox()
        hboxTweet = gtk.HBox()
        hbox0 = gtk.HBox()
        hbox1 = gtk.HBox()
        hbox2 = gtk.HBox()
        hbox3 = gtk.HBox()
        #empty the dynamic buttons

        #add avatar image and the tweet selected
        # then all the associated action buttons
        #standard buttons
        #reply
        reply = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
        reply.set_title('Reply')
        reply.connect("clicked", self.controller.replyTo)
        hbox0.pack_start(reply, expand=True)
        #reply_all
        reply_all = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
        reply_all.set_title('Reply All')
        reply_all.connect("clicked", self.controller.replyAll)
        hbox0.pack_start(reply_all, expand=True)
        retweet = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
        retweet.set_title('Retweet')
        retweet.connect("clicked", self.controller.reTweet)
        hbox0.pack_start(retweet, expand=True)
        #direct message
        dm = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
        dm.set_title('Direct Message')
        dm.connect("clicked", self.controller.directMessage)
        hbox0.pack_start(dm, expand=True)

        #get the selection
        treeselection = self.treeview.get_selection()
        select1, select2 = treeselection.get_selected_rows()
        #entry1, entry2 = self.treeview.get_selection().get_selected()
        #we might one day have more than on element selected, for now we get 1 row
        try:
            #we want to scroll to the right place in pannable area, *after* we've shown conrolls
            #so the x/y need to be in the right scope
            selectedItem = None
            if select2 != None:

                for item in select2[0]:
                    #bring the selected item to the top of the view

                    #we want to access field 2 (0,1,2) which has our Tweet in it
                    entry = select1.get_value(select1.get_iter(item), 2)
                    #and we might as well list the person who provided the url
                    name = select1.get_value(select1.get_iter(item), 0)
                    picture = select1.get_value(select1.get_iter(item), 9)
                    picture = picture.scale_simple(90, 90, gtk.gdk.INTERP_BILINEAR)
                    pic = gtk.Image()
                    pic.set_from_pixbuf(picture)
                    hboxTweet.pack_start(pic, expand=False)
                    label = gtk.Label()
                    label.set_text(select1.get_value(select1.get_iter(item), 10))
                    label.set_use_markup(True)
                    label.set_line_wrap(True)
                    label.set_selectable(True)

                    hboxTweet.pack_start(label, expand=True)

                    id = select1.get_value(select1.get_iter(item), 4)
                    selectedItem = item
                    #exclude logged in user from the reply_all string
                    if (re.search(self.controller.activeAccount.getUsername().lower(), name.lower())):
                        self.reply_all = ""
                    else:
                        self.reply_all = name
                    if re.search("http", entry):
                        #convert the string to chunks deliniated on space (we assume the url 
                        #has spaces around it 
                        L = string.split(entry)
                        for word in L :
                            #find the 'word' which is our url
                            if re.search("http", word):
                                url = word

                                b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
                                b.set_title('Open Url')
                                b.set_value(url)
                                b.connect("clicked", self.controller.openBrowser, url)
                                hbox1.pack_start(b, expand=True)
                    if re.search("@", entry):
                        L = string.split(entry)
                        for word in L :
                            if re.search("@", word):
                                user = word
                                if (re.search(self.controller.activeAccount.getUsername().lower(), user.lower())):
                                    print "excluding self from reply all"
                                else:
                                    self.controller.reply_all = self.controller.reply_all + " " + user
                                if (user != name):

                                    b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                                    b.set_title('Follow')
                                    b.set_value(user)
                                    b.connect("clicked", self.controller.activeAccount.FollowUser, user)
                                    hbox1.pack_start(b, expand=True)
                                    b2 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)

                                    b2.set_title(user)
                                    b2.connect("clicked", self.controller.tweetAtUser, user)
                                    hbox1.pack_start(b2, expand=True)
                                    b3 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)

                                    b3.set_title('History')
                                    b3.set_value(user)
                                    b3.connect("clicked", self.controller.showHist, user)
                                    hbox1.pack_start(b3, expand=True)


                    #rect = self.treeview.get_background_area(item, self.tvctweet)
                    #x, y = self.treeview.convert_bin_window_to_tree_coords(rect.x, rect.y)
                #only provide a follow button if we aren't in our own timeline (in which case we're already following
                curView = self.getCurrentView()
                if (curView != self.TIMELINE_VIEW):
                    followButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                          hildon.BUTTON_ARRANGEMENT_VERTICAL)
                    followButton.set_title('Follow')
                    followButton.set_value(name)
                    followButton.connect("clicked", self.controller.activeAccount.FollowUser, name)
                    hbox2.pack_start(followButton, expand=True)
                unfollowButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                unfollowButton.set_title('unFollow')
                unfollowButton.set_value(name)
                unfollowButton.connect("clicked", self.controller.activeAccount.UnFollowUser, name)
                hbox2.pack_start(unfollowButton, expand=True)
                histButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                histButton.set_title('History')
                histButton.set_value(name)
                histButton.connect("clicked", self.controller.showHist, name)
                hbox2.pack_start(histButton, expand=True)

                atuser = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                atuser.set_title(name)
                atuser.connect("clicked", self.controller.tweetAt, name)
                hbox2.pack_start(atuser, expand=True)

                translate = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
                translate.set_title("Translate")
                translate.connect("clicked", self.controller.Translate, entry)
                hbox3.pack_start(translate, expand=True)


                self.controller.reply_to = id
                self.controller.reply_to_name = name

                self.controller.retweettext = entry
                self.controller.retweetname = name
                self.controller.retweetid = id

        except IndexError:
            print "nothing selected"

        vbox1.pack_start(hboxTweet, expand=False)
        vbox1.pack_start(hbox0, expand=False)
        vbox1.pack_start(hbox1, expand=False)
        vbox1.pack_start(hbox2, expand=False)
        vbox1.pack_start(hbox3, expand=False)
        pannedArea.add_with_viewport(vbox1)
        win.add(pannedArea)
        win.show_all()


    def scrollToItem(self, item):
        try:
            rect = self.treeview.get_background_area(item, self.tvctweet)
            x, y = self.treeview.convert_bin_window_to_tree_coords(rect.x, rect.y)

            self.builder.get_object("pannableArea").scroll_to(x, y + 100)
        except TypeError:
            print "couldn't scroll to item"

    def scrollToSelectedTweet(self):
        #get the selection
        self.scrollToItem(self.getSelectedTweet())

    def getSelectedTweet(self):
        #get the selection
        try:
            treeselection = self.treeview.get_selection()
            select1, select2 = treeselection.get_selected_rows()
            if select2 != None:
               for item in select2[0]:
                    return item
        except IndexError:
            print "probably nothing selected"
            return None

    def dismissActionButtons(self, *args):
        self.builder.get_object("hbuttonbox-act1").hide_all()
        self.builder.get_object("hbuttonbox-act2").hide_all()
        self.builder.get_object("hbuttonbox-act3").hide_all()
        self.builder.get_object("hbuttonbox-act4").hide_all()
        self.builder.get_object("hbox2").show_all()
    def hideActionButtons(self):
        stack = hildon.WindowStack.get_default()
        stack.pop(1)
        #self.dismissActionButtons()
    def showActionButtons(self):
        self.builder.get_object("hbuttonbox-act1").show_all()
        self.builder.get_object("hbuttonbox-act2").show_all()
        self.builder.get_object("hbuttonbox-act3").show_all()
        self.builder.get_object("hbuttonbox-act4").show_all()
        self.builder.get_object("hbox2").show_all()
    def showBottomBar(self):
        self.builder.get_object("hbox1").show_all()
    def hideBottomBar(self):
        self.builder.get_object("hbox1").hide_all()
    def showTabs(self):
        self.builder.get_object("hbox2").show_all()
    def hideTabs(self):
        self.builder.get_object("hbox2").hide_all()

    def  gtk_widget_hide(self, widget, *args):
        widget.hide_all()
        #widget.destroy()

    def reparent_loc(self, widget, newParent):
        widget.reparent(newParent)

    def switchViewTo(self, widget, type):
        if (self.getCurrentView() == self.SEARCH_VIEW):
            #switching out of search view, save search terms and reset text box
            self.controller.search_terms = self.tweetText.get_text()
            self.tweetText.set_text("")
        if (re.search("timeline", type)):
            self.setActiveListStore(self.controller.activeAccount.getTimeline())
            self.setCurrentView(self.TIMELINE_VIEW)
            self.window.set_title(self.serviceName + " - timeline")
            #ensure selected view has active button image
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            #set all othe rbuttons into deactivated mode
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("direct", type)):
            self.setActiveListStore(self.controller.activeAccount.getDmsList())
            self.setCurrentView(self.DM_VIEW)

            self.window.set_title(self.serviceName + " - direct messages")
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            #set all othe rbuttons into deactivated mode
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("mentions", type)):
            self.setActiveListStore(self.controller.activeAccount.getMentionsList())
            self.setCurrentView(self.MENTIONS_VIEW)

            self.window.set_title(self.serviceName + " - mentions")
            #ensure selected view has active button image
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention.png")
            menImage.show()
            mentionsButton.set_image(menImage)

            #set all othe rbuttons into deactivated mode
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("public", type)):
            self.setActiveListStore(self.controller.activeAccount.getPublicList())
            self.setCurrentView(self.PUBLIC_VIEW)

            self.window.set_title(self.serviceName + " - public")
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            #set all othe rbuttons into deactivated mode
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("trends", type)):
            self.setActiveListStore(self.controller.activeAccount.getTrendsList())
            self.setCurrentView(self.TRENDS_VIEW)

            self.window.set_title(self.serviceName + " - trends")
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            #set all othe rbuttons into deactivated mode
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("friends", type)):
            self.setActiveListStore(self.controller.activeAccount.getFriendsList())
            self.setCurrentView(self.FRIENDS_VIEW)

            self.window.set_title(self.serviceName + " - friends")
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            #set all othe rbuttons into deactivated mode
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("search", type)):
            self.setActiveListStore(self.controller.activeAccount.getSearchList())
            self.setCurrentView(self.SEARCH_VIEW)
            self.tweetText.set_text(self.controller.search_terms)

            self.window.set_title(self.serviceName + " - search")
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search.png")
            serImage.show()
            searchButton.set_image(serImage)
            #set all othe rbuttons into deactivated mode
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory_off.png")
            userImage.show()
            userButton.set_image(userImage)
        elif (re.search("user", type)):
            self.setActiveListStore(self.controller.activeAccount.getUserHistoryList())
            self.setCurrentView(self.USERHIST_VIEW)

            self.window.set_title(self.serviceName + " - User History")
            userButton = self.builder.get_object("userhistory")
            userImage = gtk.Image()
            userImage.set_from_file("/opt/witter/icons/userHistory.png")
            userImage.show()
            userButton.set_image(userImage)
            #set all othe rbuttons into deactivated mode
            searchButton = self.builder.get_object("search")
            serImage = gtk.Image()
            serImage.set_from_file("/opt/witter/icons/search_off.png")
            serImage.show()
            searchButton.set_image(serImage)
            timelineButton = self.builder.get_object("timeline")
            tlImage = gtk.Image()
            tlImage.set_from_file("/opt/witter/icons/timeline_off.png")
            tlImage.show()
            timelineButton.set_image(tlImage)
            mentionsButton = self.builder.get_object("mentions")
            menImage = gtk.Image()
            menImage.set_from_file("/opt/witter/icons/mention_off.png")
            menImage.show()
            mentionsButton.set_image(menImage)
            dmsButton = self.builder.get_object("direct messages")
            dmImage = gtk.Image()
            dmImage.set_from_file("/opt/witter/icons/dm_off.png")
            dmImage.show()
            dmsButton.set_image(dmImage)
            friendsButton = self.builder.get_object("friends")
            friendImage = gtk.Image()
            friendImage.set_from_file("/opt/witter/icons/friends_off.png")
            friendImage.show()
            friendsButton.set_image(friendImage)
            publicButton = self.builder.get_object("public")
            pubImage = gtk.Image()
            pubImage.set_from_file("/opt/witter/icons/public_off.png")
            pubImage.show()
            publicButton.set_image(pubImage)
            trendsButton = self.builder.get_object("trends")
            trendImage = gtk.Image()
            trendImage.set_from_file("/opt/witter/icons/trends_off.png")
            trendImage.show()
            trendsButton.set_image(trendImage)

        self.builder.get_object("hbox1").show_all()
        self.builder.get_object("hbox2").show_all()

    def switchView(self, widget):
        #switches the active liststore to display what the user wants
        print widget
        type = widget.get_label()
        print type
        self.switchViewTo(widget, type)


    def CharsRemaining(self, widget):
         self.builder.get_object("hbox1").show_all()
         tweet = self.builder.get_object("TweetText").get_text()
         counter = self.builder.get_object("Counter")
         if (140 - len(tweet) < 0):
             counter.set_text("<span foreground=\"red\">" + (str(140 - len(tweet))) + "</span>")
         else:
             counter.set_text("<span foreground=\"white\">" + (str(140 - len(tweet))) + "</span>")
         counter.set_use_markup(True)


    def about(self, widget, *args):
        dlg = gtk.AboutDialog()
        dlg.set_version(self.version)
        dlg.set_name("Witter")
        #"Marcus Wikstrm (logo)"
        dlg.set_authors(["Daniel Would (programmer)", u"Marcus Wikstrￃﾶm (logo)"])
        dlg.set_website("Homepage : http://danielwould.wordpress.com/witter/\nBugtracker : http://garage.maemo.org/projects/witter")
        def close(w, res):
            if res == gtk.RESPONSE_CANCEL:
                w.hide()
        dlg.connect("response", close)
        dlg.show()

    def showBusy(self, increment, *args):
        #increment might be +1 or -1 to take the counter up or down
        self.busyCounter = self.busyCounter + increment
        print "running tasks: " + str(self.busyCounter)
        if (self.busyCounter > 0):
            #at least one thing running
            hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
        else:
            #no more tasks busy
            hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
            #in case we missed it somewhere, no longer getting Tweets
            self.gettingTweets = False

        return


    def gesture(self, widget, direction, startx, starty):
        self.builder.get_object("hbuttonbox-act1").hide_all()
        self.builder.get_object("hbuttonbox-act2").hide_all()
        self.builder.get_object("hbuttonbox-act3").hide_all()
        self.builder.get_object("hbuttonbox-act4").hide_all()
        #get the selected tweet for the view we're in and scroll to what
        #we had selecte last time we were in the view we're switching to
        widget.scroll_to(0, 0)
        print "scroll to last known position"
        curView = self.getCurrentView()
        if (direction == 3):
            if (curView == self.TIMELINE_VIEW):
                self.switchViewTo(self.treeview, "user")
            elif (curView == self.DM_VIEW):
                self.switchViewTo(self.treeview, "mentions")
            elif (curView == self.MENTIONS_VIEW):
                self.switchViewTo(self.treeview, "timeline")
            elif (curView == self.PUBLIC_VIEW):
                self.switchViewTo(self.treeview, "trends")
            elif (curView == self.TRENDS_VIEW):
                self.switchViewTo(self.treeview, "friends")
            elif (curView == self.FRIENDS_VIEW):
                self.switchViewTo(self.treeview, "search")
            elif (curView == self.SEARCH_VIEW):
                self.switchViewTo(self.treeview, "direct")
            elif (curView == self.USERHIST_VIEW):
                self.switchViewTo(self.treeview, "public")


        if (direction == 2):
            if (curView == self.TIMELINE_VIEW):
                self.switchViewTo(self.treeview, "mentions")
            elif (curView == self.DM_VIEW):
                self.switchViewTo(self.treeview, "search")
            elif (curView == self.MENTIONS_VIEW):
                self.switchViewTo(self.treeview, "direct")
            elif (curView == self.PUBLIC_VIEW):
                self.switchViewTo(self.treeview, "user")
            elif (curView == self.TRENDS_VIEW):
                self.switchViewTo(self.treeview, "public")
            elif (curView == self.FRIENDS_VIEW):
                self.switchViewTo(self.treeview, "trends")
            elif (curView == self.SEARCH_VIEW):
                self.switchViewTo(self.treeview, "friends")
            elif (curView == self.USERHIST_VIEW):
                self.switchViewTo(self.treeview, "timeline")

    def scrolling (self, widget, direction, startx, starty):
        pass
        #decided to try keeping top row of buttons at all times
        #self.builder.get_object("hbox1").hide_all()
        #self.builder.get_object("hbox2").hide_all()
        #self.hideActionButtons()
        #self.window.fullscreen ()
        #fullscrButton = self.builder.get_object("fullscr")
        #image = gtk.Image()
        #image.set_from_file("/opt/witter/icons/unfullscr.png")
        #image.show()
        #fullscrButton.set_image(image)

    def top_colour_changed(self, widget):
        color = widget.get_color()
        print "Current color is: (red=%s, green=%s, blue=%s, pixel=%s)" % (color.red, color.green, color.blue, color.pixel)
        self.cell.set_property('backgroundt_r', color.red)
        self.cell.set_property('backgroundt_g', color.green)
        self.cell.set_property('backgroundt_b', color.blue)
        self.cell.set_property('backgroundt_p', color.pixel)
        self.bg_top_color = color
    def bottom_colour_changed(self, widget):
        color = widget.get_color()
        print "Current color is: (red=%s, green=%s, blue=%s, pixel=%s)" % (color.red, color.green, color.blue, color.pixel)
        self.cell.set_property('backgroundb_r', color.red)
        self.cell.set_property('backgroundb_g', color.green)
        self.cell.set_property('backgroundb_b', color.blue)
        self.cell.set_property('backgroundb_p', color.pixel)
        self.bg_bottom_color = color

    def show_props_window(self, widget):
       # a stacked window to contain all properties
       win = hildon.StackableWindow()
       win.set_title("Properties")

       pannedArea = hildon.PannableArea()
       #
       #define the property objects
       #


       bitlyId = gtk.Label("Bit.ly userid")
       bitlyEntry = gtk.Entry()
       bitlyEntry.set_text(self.controller.bitlyusername)
       bitlyEntry.connect("changed", self.controller.setBitlyuid)
       bitlyPwd = gtk.Label("Bit.ly apikey")

       bitlyPwdEntry = gtk.Entry()
       bitlyPwdEntry.set_text(self.controller.bitlyapikey)
       bitlyPwdEntry.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL)
       bitlyPwdEntry.connect("changed", self.controller.setBitlyapiki)
       #tllabel = gtk.Label("Timeline refresh interval")
       tl_picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       tl_picker_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       tl_picker_button.set_title("Set timeline refresh interval")
       tlNumEd = self.create_refresh_interval_selector(self.controller.timelineRefreshInterval)
       tl_picker_button.set_selector(tlNumEd)
       tl_picker_button.connect("value-changed", self.on_tl_refresh_value_changed)

       men_picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       men_picker_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       men_picker_button.set_title("Set mention refresh interval")
       menNumEd = self.create_refresh_interval_selector(self.controller.mentionsRefreshInterval)
       men_picker_button.set_selector(menNumEd)
       men_picker_button.connect("value-changed", self.on_men_refresh_value_changed)

       dm_picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       dm_picker_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       dm_picker_button.set_title("Set dm refresh interval")
       dmNumEd = self.create_refresh_interval_selector(self.controller.DMsRefreshInterval)
       dm_picker_button.set_selector(dmNumEd)
       dm_picker_button.connect("value-changed", self.on_dm_refresh_value_changed)
       pub_picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       pub_picker_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       pub_picker_button.set_title("Set public refresh interval")
       pubNumEd = self.create_refresh_interval_selector(self.controller.publicRefreshInterval)
       pub_picker_button.set_selector(pubNumEd)
       pub_picker_button.connect("value-changed", self.on_pub_refresh_value_changed)

       search_picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       search_picker_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       search_picker_button.set_title("Set search refresh interval")
       searchNumEd = self.create_refresh_interval_selector(self.controller.searchRefreshInterval)
       search_picker_button.set_selector(searchNumEd)
       search_picker_button.connect("value-changed", self.on_search_refresh_value_changed)



        # Create touch selector
       selector = self.create_theme_selector()

       # Create a picker button
       picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       picker_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       picker_button.set_title("Select a theme")

       # Attach the touch selector to the picker button
       picker_button.set_selector(selector)

       # Attach callback to the "value-changed" signal
       picker_button.connect("value-changed", self.on_picker_value_changed)


       vbox = gtk.VBox(False, 0)
       vbox.pack_start(bitlyId, True, True, 0)
       vbox.pack_start(bitlyEntry, True, True, 0)
       vbox.pack_start(bitlyPwd, True, True, 0)
       vbox.pack_start(bitlyPwdEntry, True, True, 0)
       vbox.pack_start(tl_picker_button, True, True, 0)
       vbox.pack_start(men_picker_button, True, True, 0)
       vbox.pack_start(dm_picker_button, True, True, 0)
       vbox.pack_start(pub_picker_button, True, True, 0)
       vbox.pack_start(search_picker_button, True, True, 0)

       vbox.pack_start(picker_button, True, True, 0)


       pannedArea.add_with_viewport(vbox)


       win.add(pannedArea)

       # This call show the window and also add the window to the stack
       win.show_all()

    def create_theme_selector(self):
       selector = hildon.TouchSelectorEntry(text=True)

       # Stock icons will be used for the example
       theme_list = ["Default"]

       match_found = False
       # Populate model
       for item in theme_list:
           selector.append_text(item)


       selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
       return selector

    def create_refresh_interval_selector(self, selected_value):
       # Create a touch selector 
       selector = hildon.TouchSelectorEntry(text=True)

       # Stock icons will be used for the example
       int_list = ["0", "5", "10", "15", "20", "25", "30", "60", "90", "120"]
       active_index = 0
       count = 0
       match_found = False
       # Populate model
       for item in int_list:
           if (re.search(item, str(selected_value))):
               match_found = True
               active_index = count
           selector.append_text(item)
           count = count + 1
       if match_found == False:
           active_index = 0
       print "setting active index to " + str(active_index)
       selector.set_active(0, active_index)
       selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
       return selector

    def on_picker_value_changed(self, button, user_data=None):
       print "Newly selected value: %s\n" % button.get_value()

    def on_tl_refresh_value_changed(self, button, user_data=None):
       self.controller.timelineRefreshInterval = int(button.get_value())

    def on_men_refresh_value_changed(self, button, user_data=None):
       self.controller.mentionsRefreshInterval = int(button.get_value())

    def on_dm_refresh_value_changed(self, button, user_data=None):
       self.controller.DMsRefreshInterval = int(button.get_value())
    def on_pub_refresh_value_changed(self, button, user_data=None):
       self.controller.publicRefreshInterval = int(button.get_value())
    def on_search_refresh_value_changed(self, button, user_data=None):
       self.controller.searchRefreshInterval = int(button.get_value())


    def showMoreButtons(self, *args):
        if (self.showingMore == True):
            self.builder.get_object("hbuttonbox-more").hide_all()
            self.showingMore = False
            extraButton = self.builder.get_object("extra-controls")
            image2 = gtk.Image()
            image2.set_from_file("/opt/witter/icons/plus.png")
            image2.show()
            extraButton.set_image(image2)
        else:
            self.builder.get_object("hbuttonbox-more").show_all()
            self.showingMore = True
            extraButton = self.builder.get_object("extra-controls")
            image2 = gtk.Image()
            image2.set_from_file("/opt/witter/icons/minus.png")
            image2.show()
            extraButton.set_image(image2)

    def promptForCredentials(self, *args):
        #dialog = self.wTree.get_widget("CredentialsDialog")
        dialog = self.builder.get_object("CredentialsDialog")
        password = self.builder.get_object("Password")
        password.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL)
        dialog.set_title("Twitter Credentials")
        dialog.connect("response", self.gtk_widget_hide)
        dialog.show_all()

    def showOauthPrompts(self):
         note = hildon.Note("confirmation", self.window, "authorise with twitter...")
         note.set_button_texts("ok", "cancel")
         note.connect("response", self.gtk_widget_hide)
         retcode = gtk.Dialog.run(note)

         if retcode == gtk.RESPONSE_OK:
            print "User pressed 'OK' button'"

         else:
            print "User pressed 'Cancel' button"
            return


         dialog = self.builder.get_object("OauthDialog")
         dialog.set_title("Twitter Credentials")
         dialog.connect("response", self.gtk_widget_hide)
         dialog.show_all()

    def getOauthPIN(self):
        '''returns the current value of the oauthDialogs PIN field'''
        return self.builder.get_object("PIN").get_text()

    def shortenUrl(self, widget, *args):
        # a stacked window to contain all properties
       win = hildon.StackableWindow()
       win.set_title("Properties")

       pannedArea = hildon.PannableArea()
       #
       #define the property objects
       #


       label = gtk.Label()#
       label.set_text("Enter URL to shorten")
       entry = gtk.Entry()

       ShortenButton = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)

       ShortenButton.set_label("Shorten URL")
       # Attach callback to clicked signal
       ShortenButton.set_name("HildonButton-finger")
       ShortenButton.connect("clicked", self.controller.getShortenedURL, entry)
       ShortenButton.show()
       label2 = gtk.Label()#
       label2.set_text("Shortened URL will be placed in tweet entry box")
       vbox = gtk.VBox(False, 0)
       vbox.pack_start(label, True, True, 0)
       vbox.pack_start(entry, True, True, 0)
       vbox.pack_start(ShortenButton, False, False, 0)
       vbox.pack_start(label2, True, True, 0)

       pannedArea.add_with_viewport(vbox)


       win.add(pannedArea)

       # This call show the window and also add the window to the stack
       win.show_all()

    def appendTweetText(self, text):
        self.tweetText.set_text(self.tweetText.get_text() + " " + text)

    def setTweetText(self, text):
        self.tweetText.set_text(text)

    def getEntryText(self):
        return self.tweetText.get_text()
    def set_title(self, title):
        self.window.set_title(title)

    def getCurrentView(self):
        return self.activeView
    def setCurrentView(self, viewId):
        self.activeView = viewId

    def setCursorAt(self, pos):
        self.tweetText.grab_focus()
        self.tweetText.set_position(pos);

    def show_account_window(self, widget):
       # a stacked window to contain all properties
       win = hildon.StackableWindow()
       win.set_title("Accounts")

       pannedArea = hildon.PannableArea()
       #
       #define the property objects
       #

       AccountTitle = gtk.Label("Accounts")
       selector = hildon.TouchSelector(text=True)

       # Stock icons will be used for the example
       account_list = self.controller.config.accountList

       match_found = False
       # Populate model
       for item in account_list:
           if (item.status == item.ACTIVE):
               selector.append_text(item.username + " - " + item.servicename + " - Active")
           else:
               selector.append_text(item.username + " - " + item.servicename)

       selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
       NewAccButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
       NewAccButton.set_title("New")
       NewAccButton.connect("clicked", self.edit_new_account)
       EditAccButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)

       EditAccButton.set_title("Edit")
       EditAccButton.connect("clicked", self.edit_selected_account, selector, self.controller.config.accountList)
       SetActiveAccButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
       SetActiveAccButton.set_title("Set Active")
       SetActiveAccButton.connect("clicked", self.setActiveAccount, selector, self.controller.config.accountList)

       DeleteAccButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
       DeleteAccButton.set_title("Delete")
       DeleteAccButton.connect("clicked", self.deleteAccount, selector, self.controller.config.accountList)

       hbox = gtk.HBox(False, 0)
       vboxright = gtk.VBox(False, 0)
       vboxleft = gtk.VBox(False, 0)
       #vbox.pack_start(AccountTitle, True, True, 0)
       vboxright.pack_start(NewAccButton, False, False, 0)
       vboxright.pack_start(EditAccButton, False, False, 0)
       vboxright.pack_start(SetActiveAccButton, False, False, 0)
       vboxright.pack_start(DeleteAccButton, False, False, 0)
       vboxleft.pack_start(AccountTitle, False, False, 0)
       vboxleft.pack_start(selector, True, True, 0)
       hbox.pack_start(vboxleft, True, True, 0)

       hbox.pack_start(vboxright, True, True, 0)
       pannedArea.add_with_viewport(hbox)


       win.add(pannedArea)

       # This call show the window and also add the window to the stack
       win.show_all()

    def edit_selected_account(self, widget, selector, accList):
        selected = selector.get_current_text()
        for account in accList:
            if (re.search(account.username, selected)):
                if(re.search(account.servicename, selected)):
                   self.edit_account_details(widget, account)


    def edit_new_account(self, widget):
        accData = account.accountdata()
        self.edit_account_details(widget, accData)
        self.controller.config.accountList.append(accData)
        self.controller.addNewAccount(accData)


    def edit_account_details(self, widget, account):

       win = hildon.StackableWindow()
       win.set_title("Edit Account - " + account.username)

       pannedArea = hildon.PannableArea()
       #
       #define the property objects
       #


       AccountTitle = gtk.Label("Account")
       selector = self.create_account_type_selector(account.servicename)

       # Create a picker button
       accType_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
                                           hildon.BUTTON_ARRANGEMENT_VERTICAL)
       accType_button.set_name("HildonButton-thumb")
       # Set a title to the button 
       accType_button.set_title("Account Type")

       # Attach the touch selector to the picker button
       accType_button.set_selector(selector)
       ServiceUrl = gtk.Entry()
       ServiceUrl.set_text(account.baseUrl)
       ServiceUrl.connect("changed", self.changed_base_url, account)
       SearchUrl = gtk.Entry()
       SearchUrl.set_text(account.searchUrl)
       SearchUrl.connect("changed", self.changed_search_url, account)
       selector.connect("changed", self.changed_account_type, ServiceUrl, SearchUrl, account)
       # Attach callback to the "value-changed" signal
       #AccountType = gtk.Label("Account Type: " + account.getServicename())

       UsernameLabel = gtk.Label("Username")
       UsernameEntry = gtk.Entry()
       UsernameEntry.connect("changed", self.changed_username, account)
       UsernameEntry.set_text(account.username)
       PasswordLabel = gtk.Label("Password")
       PasswordEntry = gtk.Entry()
       PasswordEntry.set_invisible_char("*")
       PasswordEntry.set_visibility(False)
       PasswordEntry.connect("changed", self.changed_password, account)

       if (account.password == None):
           PasswordEntry.set_text("Not Set")
       else:
           PasswordEntry.set_text(account.password)
       hbox = gtk.HBox(False, 0)
       vbox = gtk.VBox(False, 0)
       vbox.pack_start(AccountTitle, True, True, 0)
       vbox.pack_start(accType_button, True, True, 0)
       vbox.pack_start(UsernameLabel, True, True, 0)
       vbox.pack_start(UsernameEntry, True, True, 0)
       vbox.pack_start(PasswordLabel, True, True, 0)
       vbox.pack_start(PasswordEntry, True, True, 0)


       if (account.servicename == "Twitter"):
           if (account.access_token == None):
               AccessTokenIndicator = gtk.Label()
               AccessTokenIndicator.set_text("Oauth Not Configured")

           else:
               AccessTokenIndicator = gtk.Label()
               AccessTokenIndicator.set_text("Oauth Configured")
           hbox.pack_start(AccessTokenIndicator, True, True, 0)
           Creds2 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                           hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
           Creds2.set_label("Configure Oauth")
           # Attach callback to clicked signal
           Creds2.connect("clicked", self.controller.configOauth, account)
           Creds2.show()
           hbox.pack_start(Creds2, False, False, 0)
           vbox.pack_start(hbox, True, True, 0)


       vbox.pack_start(ServiceUrl, True, True, 0)
       vbox.pack_start(SearchUrl, True, True, 0)



       pannedArea.add_with_viewport(vbox)


       win.add(pannedArea)
       win.connect("destroy", self.finished_account_edit)
       # This call show the window and also add the window to the stack
       win.show_all()


    def create_account_type_selector(self, type):
       selector = hildon.TouchSelector(text=True)

       # Stock icons will be used for the example
       type_list = ["Twitter", "Identi.ca", "Other"]
       count = 0
       match_found = False
       # Populate model
       for item in type_list:
           selector.append_text(item)
           if (item == type):
               selector.set_active(0, count)
           else:
               count = count + 1

       selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
       return selector

    def changed_account_type(self, selector, user_data, serviceUrl, searchUrl, account):
       selected = selector.get_current_text()
       if (selected == "Twitter"):
           serviceUrl.set_text("https://twitter.com/")
           searchUrl.set_text("https://search.twitter.com/")
           account.servicename = "Twitter"
       elif(selected == "Identi.ca"):
           account.servicename = "Identi.ca"
           serviceUrl.set_text("http://identi.ca/api/")
           searchUrl.set_text("http://identi.ca/api/")
       else:
           account.servicename = "Other"
           serviceUrl.set_text("Enter Twitter compatible base url")
           searchUrl.set_text("Enter Twitter Compatible search url")
       account.baseUrl = serviceUrl.get_text()
       account.searchUrl = searchUrl.get_text()


    def changed_username(self, widget, account):
        account.username = widget.get_text()

    def changed_password(self, widget, account):
        account.password = widget.get_text()

    def changed_base_url(self, widget, account):
        account.baseUrl = widget.get_text()

    def changed_search_url(self, widget, account):
        account.searchUrl = widget.get_text()

    def finished_account_edit(self, widget):
        stack = hildon.WindowStack.get_default()
        stack.pop(1)
        self.show_account_window(widget)

    def setActiveAccount(self, widget, selector, accList):
        selected = selector.get_current_text()
        for account in accList:
            if (re.search(account.username, selected)):
                if(re.search(account.servicename, selected)):
                   account.status = account.ACTIVE
                   self.controller.setActiveAccount(account)
                else:
                   account.status = account.INACTIVE
            else:
                account.status = account.INACTIVE
        stack = hildon.WindowStack.get_default()
        stack.pop(1)
        self.show_account_window(widget)


    def deleteAccount(self, widget, selector, accList):
        selected = selector.get_current_text()
        for account in accList:
            if (re.search(account.username, selected)):
                if(re.search(account.servicename, selected)):
                   if (account.status == account.ACTIVE):
                       #can't delete the active account
                       note = osso.SystemNote(self.controller.osso_c)
                       note.system_note_dialog("Can't delete the active account")
                   else:
                       self.controller.deleteAccount(account)

        stack = hildon.WindowStack.get_default()
        stack.pop(1)
        self.show_account_window(widget)

    def setWindowTitlePrefix(self, prefix):
        self.serviceName = prefix

    def setWindowTitle(self, title):
        self.window.set_title(title)

    def reload_account_window(self, widget, account):
        stack = hildon.WindowStack.get_default()
        stack.pop(1)
        self.edit_account_details(widget, account)



